@@ -24,16 +24,16 @@ options = {
 }
 
 boptions = {
-  "iconfile": "./resources/ico/carla.icns"
+  "iconfile": "./resources/ico/carla%s.icns" % ("-control" if SCRIPT_NAME == "Carla-Control" else "")
 }
 
 if SCRIPT_NAME in ("Carla", "Carla-Control"):
-  boptions["custom_info_plist"] = "./data/macos/%s_Info.plist" % SCRIPT_NAME
+  boptions["custom_info_plist"] = "./data/macos/%s.plist" % SCRIPT_NAME
 
 setup(name = "Carla",
       version = VERSION,
       description = "Carla Plugin Host",
       options = {"build_exe": options, "bdist_mac": boptions},
-      executables = [Executable("./source/frontend/%s.pyw" % SCRIPT_NAME)])
+      executables = [Executable("./source/frontend/%s" % SCRIPT_NAME)])
 
 # ------------------------------------------------------------------------------------------------------------
