@@ -318,7 +318,7 @@ extern void carla_register_native_plugin(const NativePluginDescriptor* desc);
 void carla_register_all_native_plugins(void);
 
 /** Get meta-data only */
-CARLA_EXPORT
+CARLA_API_EXPORT
 const NativePluginDescriptor* carla_get_native_plugins_data(uint32_t* count);
 
 /* ------------------------------------------------------------------------------------------------------------ */
