@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -30,12 +30,8 @@
 // GCC
 #if JUCE_GCC
 
- #if (__GNUC__ * 100 + __GNUC_MINOR__) < 407
-  #error "JUCE requires GCC 4.7 or later"
- #endif
-
- #if ! (__cplusplus >= 201103L || defined (__GXX_EXPERIMENTAL_CXX0X__))
-  #error "JUCE requires that GCC has C++11 compatibility enabled"
+ #if (__GNUC__ * 100 + __GNUC_MINOR__) < 500
+  #error "JUCE requires GCC 5.0 or later"
  #endif
 
  #ifndef JUCE_EXCEPTIONS_DISABLED
@@ -44,7 +40,7 @@
   #endif
  #endif
 
- #define JUCE_CXX14_IS_AVAILABLE ((__cplusplus >= 201402L) || ((__GNUC__ * 100 + __GNUC_MINOR__) >= 409 && (__cplusplus >= 201300L)))
+ #define JUCE_CXX14_IS_AVAILABLE (__cplusplus >= 201402L)
  #define JUCE_CXX17_IS_AVAILABLE (__cplusplus >= 201703L)
 
 #endif
@@ -53,8 +49,8 @@
 // Clang
 #if JUCE_CLANG
 
- #if (__clang_major__ < 3) || (__clang_major__ == 3 && __clang_minor__ < 3)
-  #error "JUCE requires Clang 3.3 or later"
+ #if (__clang_major__ < 3) || (__clang_major__ == 3 && __clang_minor__ < 4)
+  #error "JUCE requires Clang 3.4 or later"
  #endif
 
  #ifndef JUCE_COMPILER_SUPPORTS_ARC
@@ -76,8 +72,8 @@
 // MSVC
 #if JUCE_MSVC
 
- #if _MSC_VER < 1900 // VS2015
-   #error "JUCE requires Visual Studio 2015 or later"
+ #if _MSC_FULL_VER < 191025017  // VS2017
+   #error "JUCE requires Visual Studio 2017 or later"
  #endif
 
  #ifndef JUCE_EXCEPTIONS_DISABLED
@@ -91,25 +87,12 @@
 #endif
 
 //==============================================================================
-// C++ library
-#if (defined (__GLIBCXX__) && __GLIBCXX__ < 20130322)
- #error "JUCE requires a C++ library containing std::atomic"
+#if ! JUCE_CXX14_IS_AVAILABLE
+ #error "JUCE requires C++14 or later"
 #endif
 
 //==============================================================================
-#if (! JUCE_MSVC) && (! JUCE_CXX14_IS_AVAILABLE)
-namespace std
-{
-    template<typename T, typename... Args>
-    unique_ptr<T> make_unique (Args&&... args)
-    {
-        return unique_ptr<T> (new T (std::forward<Args> (args)...));
-    }
-}
-#endif
-
-//==============================================================================
-#if ! DOXYGEN
+#ifndef DOXYGEN
  // These are old flags that are now supported on all compatible build targets
  #define JUCE_COMPILER_SUPPORTS_OVERRIDE_AND_FINAL 1
  #define JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES 1
@@ -118,3 +101,9 @@ namespace std
  #define JUCE_DELETED_FUNCTION = delete
  #define JUCE_CONSTEXPR constexpr
 #endif
+
+#if JUCE_CXX17_IS_AVAILABLE
+ #define JUCE_NODISCARD [[nodiscard]]
+#else
+ #define JUCE_NODISCARD
+#endif
