@@ -1,6 +1,6 @@
 /*
  * Carla Plugin Host
- * Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
 
 #include "CarlaNativePlugin.h"
 
-#define CARLA_PLUGIN_EXPORT
+#define CARLA_PLUGIN_BUILD
 
 #include "carla-native-plugin.cpp"
 
