@@ -1,6 +1,6 @@
 /*
  * Carla macOS utils
- * Copyright (C) 2018-2021 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2018-2022 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,10 @@
 #endif
 
 #include "CarlaBackend.h"
- 
+
+// don't include Foundation.h here
+typedef struct __CFBundle* CFBundleRef;
+
 CARLA_BACKEND_START_NAMESPACE
 
 // --------------------------------------------------------------------------------------------------------------------
@@ -43,6 +46,8 @@ const char* findBinaryInBundle(const char* const bundleDir);
  */
 bool removeFileFromQuarantine(const char* const filename);
 
+// --------------------------------------------------------------------------------------------------------------------
+
 /*
  * ...
  */
@@ -57,6 +62,19 @@ private:
 
 // --------------------------------------------------------------------------------------------------------------------
 
+struct BundleLoader {
+    BundleLoader();
+    ~BundleLoader();
+    bool load(const char* const filename);
+    CFBundleRef getRef() const noexcept;
+
+private:
+    struct PrivateData;
+    PrivateData* const pData;
+};
+
+// --------------------------------------------------------------------------------------------------------------------
+
 CARLA_BACKEND_END_NAMESPACE
 
 #endif // CARLA_MAC_UTILS_HPP_INCLUDED
